package com.xkrs.controller;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.common.tool.TokenUtil;
import com.xkrs.dao.ProjectOverviewDao;
import com.xkrs.model.entity.ProjectOverview;
import com.xkrs.model.entity.SysUserEntity;
import com.xkrs.model.qo.ProjectQo;
import com.xkrs.model.qo.ProjectUpdateQo;
import com.xkrs.service.ProjectOverviewService;
import com.xkrs.service.SysUserService;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

/**
 * @author XinYi Song
 */
@RestController
public class ProjectOverviewController {

    @Resource
    private ProjectOverviewService projectOverviewService;

    @Resource
    private SysUserService sysUserService;

    @Resource
    private ProjectOverviewDao projectOverviewDao;

    /**
     * 导入遥感的excel表
     * @param file
     * @return
     * @throws Exception
     */
    @PostMapping("/importRsProjectExcel")
    public String importRsProjectExcel(MultipartFile file) throws Exception {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        projectOverviewService.importRsProjectExcel(file);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"导入成功",locale);
    }

    /**
     * 导入视觉的excel表
     * @param file
     * @return
     * @throws Exception
     */
    @PostMapping("/importCvProjectExcel")
    public String importCvProjectExcel(MultipartFile file) throws Exception {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        projectOverviewService.importCvProjectExcel(file);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"导入成功",locale);
    }

    /**
     * 动态多条件查询项目的信息
     * @param projectNumber
     * @param projectName
     * @return
     */
    @GetMapping("/selectProjectByDynamic")
    @PreAuthorize("hasAnyAuthority('auth_administor','auth_general_user')")
    public String selectProjectByDynamic(@RequestParam("projectNumber") String projectNumber,@RequestParam("projectName") String projectName,@RequestHeader(value="Authorization") String token){
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserByUserName = sysUserService.getSysUserByUserName(tokenUserName);
        if(sysUserByUserName == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        List<ProjectOverview> projectOverviews = projectOverviewService.selectProjectByDynamic(projectNumber, projectName);
        if(projectOverviews == null || projectOverviews.size() == 0){
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE,"暂时还没有项目信息",locale);
        }else {
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS,projectOverviews,locale);
        }
    }

    /**
     * 新增项目
     * @param projectQo
     * @param token
     * @return
     */
    @PostMapping("/insertProjectOverview")
    @PreAuthorize("hasAnyAuthority('auth_administor')")
    public String insertProjectOverview(@RequestBody ProjectQo projectQo,@RequestHeader(value="Authorization") String token){
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserByUserName = sysUserService.getSysUserByUserName(tokenUserName);
        if(sysUserByUserName == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        ProjectOverview byProjectName = projectOverviewDao.findByProjectName(projectQo.getProjectName());
        if(byProjectName != null){
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"该项目已存在",locale);
        }
        ProjectOverview byProjectNumber = projectOverviewDao.findByProjectNumber(projectQo.getProjectNumber());
        if(byProjectNumber != null){
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"项目编号重复",locale);
        }
        ProjectOverview projectOverview = projectOverviewService.insertProjectOverview(projectQo);
        if(projectOverview != null){
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"添加成功",locale);
        }else {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"添加失败",locale);
        }
    }

    /**
     * 更改项目信息
     * @param projectUpdateQo
     * @param token
     */
    @PostMapping("/updateProjectOverview")
    @PreAuthorize("hasAnyAuthority('auth_administor')")
    public String updateProjectOverview(@RequestBody ProjectUpdateQo projectUpdateQo,@RequestHeader(value="Authorization") String token){
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserByUserName = sysUserService.getSysUserByUserName(tokenUserName);
        if(sysUserByUserName == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        projectOverviewService.updateProjectOverview(projectUpdateQo);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"修改成功",locale);
    }

    /**
     * 根据项目编号删除项目
     * @param map
     * @param token
     * @return
     */
    @PostMapping("/deleteByProjectNumber")
    @PreAuthorize("hasAnyAuthority('auth_administor','auth_general_user')")
    public String deleteByProjectNumber(@RequestBody Map map,@RequestHeader(value="Authorization") String token){
        String projectNumber = (String) map.get("projectNumber");
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserByUserName = sysUserService.getSysUserByUserName(tokenUserName);
        if(sysUserByUserName == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        projectOverviewService.deleteByProjectNumber(projectNumber);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"删除成功",locale);
    }
}
